﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;

namespace IMSES_Template.Model
{

    public abstract class BaseEntity
    {
        /// <summary>
        /// Checks if the object is valid, i.e. that all properties marked with [Required] are actually initialized and filled, 
        /// and throws an exception with the porperties that aren't.
        /// </summary>
        internal void CheckRequired(string syncId)
        {
            var props = new StringBuilder(string.Empty);
            foreach (var property in 
                    from property in GetType().GetProperties() 
                    from requiredAttribute in property.GetCustomAttributes(true).OfType<RequiredAttribute>().Where(req => !req.IsValid(property.GetValue(this, null))) 
                    select property)
            {
                props.Append(property.Name + ", ");
            }

            if (props.Length > 0)
            {
                throw new ValidationException(
                    string.Format("({0}) The following required properties have not been filled in: {1}",
                                  syncId, props.Remove(props.Length - 2, 2)));
            }
        }

    }

    [AttributeUsage(AttributeTargets.Property | AttributeTargets.Field, AllowMultiple = false)]
    public class RequiredAttribute : ValidationAttribute
    {
        public override bool IsValid(Object testObject)
        {
            return (testObject != null && testObject.ToString().Trim().Length > 0);
        }
    }

    public class KeyPair
    {
        public string OldKey;
        public string NewKey;
    }

    public class MembershipPair
    {
        public string UserSyncKey;
        public string HierarchySyncId;
        public HierarchyRole Role;
    }

    public class OperationResult
    {
        public ResultStatus Status;
        public string Identifier;
        public string Message;

        public OperationResult(ResultStatus status, string identifier, string message)
        {
            Status = status;
            Identifier = identifier;
            Message = message;
        }

    }

    public class BatchOperationResult
    {
        public ResultStatus Status;
        public List<OperationResult> ErrorMessages;
        public List<OperationResult> WarningMessages;

        public BatchOperationResult(ResultStatus status, List<OperationResult> errorMessages, List<OperationResult> warningMessages)
        {
            Status = status;
            ErrorMessages = errorMessages;
            WarningMessages = warningMessages;
        }
    }

}
